require "rubygems"

require "prawn"
require "prawn/measurement_extensions"

GRAY        = 'aaaaaa'
ORCHID      = '9932CD'
BLACK       = '000000'

class Prawn::Document
  def texts(lines, *parameters)
    lines.each do |line|
      text line, *parameters
    end
  end

  def with_shadow(parameters = {})
    shadow_color = parameters[:color] || GRAY
    depth = parameters[:depth] || 0.5.mm

    original_cursor = cursor
    original_fill_color = fill_color

    fill_color shadow_color

    move_down depth
    indent depth do
      yield
    end

    move_cursor_to original_cursor
    fill_color original_fill_color
    yield
  end

  def text_with_shadow(*text_args)
    with_shadow do
      text *text_args
    end
  end
end

class Handout
  DEFAULT_TRAINERS = [
      ["Rob Westgeest", "Westgeest-Consultancy",
       "Gondelstraat 2", "5017 CK  Tilburg, NL", "+31 6 45 776 328", "rob@qwan.eu"],
      ["Marc Evers", "Piecemeal Growth", "Delftseschans 18",
       "3432 TD  Nieuwegein, NL", "+31 6 4455 0003", "marc@qwan.eu"],
      ["Willem van den Ende", "Living Software B.V.", "Spilmanstraat 25",
       "5645 JE  Eindhoven, NL", "+31 6 4130 6965", "willem@qwan.eu"]]

  SERIF_FONT      = "Times-Roman"
  COMIC_SANS_FONT = "#{Prawn::BASEDIR}/data/fonts/comicsans.ttf"


  def self.create(parameters)
    raise "need course name" unless parameters[:course]
    raise "need course dates" unless parameters[:date]
    raise "need location/client" unless parameters[:location]
    raise "need titles and goals for tabs" unless parameters[:tabs]

    handout = Handout.new parameters
    handout.create_frontpage
    handout.create_toc
    handout.create_tabs
  end

  def initialize arguments = {}
    @trainers = arguments[:trainers] || DEFAULT_TRAINERS
    @course = arguments[:course] || ""
    @date = arguments[:date] || ""
    @location = arguments[:location] || ""
    @tabs = arguments[:tabs] || [[]]
  end

  def header(pdf)
    pdf.font COMIC_SANS_FONT
    pdf.fill_color GRAY
    
    pdf.with_shadow :color => BLACK do
      pdf.text @course, :size => 20, :align => :center
    end
  end

  def footer(pdf, text)
    pdf.move_cursor_to(10.mm)
    pdf.text text, :align => :center
  end

  def new_page(pdf_file, footer_text = "")
    Prawn::Document.generate(pdf_file, :page_size => 'A4',
        :left_margin => 15.mm,
        :right_margin => 15.mm,
        :top_margin => 20.mm,
        :bottom_margin => 15.mm
    ) do | pdf |
      yield pdf

      footer(pdf, footer_text)
    end
  end

  def add_trainer_info(pdf)
    pdf.column_box([0, pdf.cursor], :columns => 3, :width => pdf.bounds.width) do
      pdf.texts @trainers[0], :size => 11, :align => :left
      pdf.bounds.move_past_bottom
      pdf.texts @trainers[1], :size => 11, :align => :center
      pdf.bounds.move_past_bottom
      pdf.texts @trainers[2], :size => 11, :align => :right
    end
  end

  def create_frontpage
    footer = "(c) 2005-2013 QWAN -Quality Without a Name\nwww.qwan.eu"

    new_page("frontpage.pdf", footer) do | pdf |
      pdf.image "#{File.dirname(__FILE__)}/qwan_logo_small.png", :position => :center

      pdf.font "Helvetica"
      pdf.text "\n\n\nCourse Handout\n\n\n\n", :size => 20, :align => :center

      pdf.fill_color ORCHID
      pdf.font COMIC_SANS_FONT
      pdf.text_with_shadow @course, :size => 36, :align => :center

      pdf.fill_color BLACK
      pdf.font "Helvetica"
      pdf.text "\n\n\n\n\n\n#{@date}", :size => 20, :align => :center

      pdf.text "#{@location}\n\n\n\n\n\n\n\n", :align => :center

      add_trainer_info(pdf)
    end
  end

  def tab_number(pdf, tab_nr)
    pdf.bounding_box [pdf.bounds.left, pdf.bounds.top], :width => 60, :height => 60 do
      pdf.text_with_shadow tab_nr, :size => 42
    end
  end

  def title_and_goal(pdf, goal, title)
    pdf.bounding_box [pdf.bounds.left + 28, pdf.bounds.top], :width => 500, :height => 60 do
      pdf.text_with_shadow title, :size => 18
      pdf.fill_color BLACK
      pdf.text goal, :size => 12
    end
  end

  def tab_entry(pdf, title, goal, tab_nr)
    pdf.bounding_box [15.mm, pdf.cursor - 5.mm], :width => 600 do
      pdf.font SERIF_FONT
      pdf.fill_color ORCHID

      tab_number(pdf, tab_nr)
      title_and_goal(pdf, goal, title)
    end
  end

  def tab_footer
    "#{@course} / #{@date}, #{@location}"
  end

  def create_toc
    new_page("toc.pdf", tab_footer) do | pdf |
      header(pdf)

      pdf.move_down 10.mm

      @tabs.each_with_index do | (title, goal), index |
        tab_entry(pdf, title, goal, (index + 1).to_s)
      end
    end
  end

  def tab(title, goal, tab_nr)
    new_page("tab#{tab_nr}.pdf", tab_footer) do |pdf|
      pdf.move_down 40.mm

      tab_entry(pdf, title, "", tab_nr)

      pdf.bounding_box [30.mm, pdf.cursor - 20.mm], :width => 140.mm do
        pdf.fill_color BLACK
        pdf.text goal, :size => 16
      end
    end
  end

  def create_tabs
    @tabs.each_with_index do | (title, goal), index |
      tab title, goal, (index+1).to_s
    end
  end
end

